import QtQuick 1.1
import "functions.js" as Lib
import Translator 1.0
import CursorShape 1.0

/* main.qml */
MouseArea {
    id: root
    width: displaySize.width
    height: displaySize.height
    hoverEnabled: true

    property variant displaySize: context.displaySize
    property variant panelSize: Lib.getPanelSize(width, height)
    property variant playlistSize: Lib.getPlaylistSize(width)
    property string imgPath: Lib.getImagesPath(width)
    property bool canHide: !playlist.visible
    property string visittext: ""
    property bool isAd: false
    property int waitForAd: 0
    property bool adCanBeDisabled: false
    property bool adCanBeVisited: false

    /* signals */
    signal commandPlay();                      // play button click
    signal commandStop(variant val);           // stop/power button click
    signal commandPrev();                      // prev button click
    signal commandNext();                      // next button click
    signal commandMute();                      // mute button click
    signal commandFullscreen();                // fullscreen button click
    signal commandVolumeSet(variant val);      // volume value change
    signal commandVolumeDown();                // volume - 1
    signal commandVolumeUp();                  // volume + 1
    signal commandPlaybackSet(variant val);    // playback value change
    signal commandCheckItem(variant index);    // check playlist item
    signal commandPlayItem(variant index);     // play playlist item
    signal commandCheckAll();                  // check all playilst items
    signal commandUncheckAll();                // uncheck all playlist items
    signal commandMouseOnPlaylist(variant val);// mouse pointer is over playlist, mouseWheel event will scroll playlist
    signal commandHidden();                    // animation to hide panel finished, hide controls window
    signal commandCanHideChanged(variant val); // can start animation to hide
    signal commandSaveCurrent();               // save button on panel clicked
    signal commandSaveItem(variant index);     // save button on playlist clicked
    signal commandSetMinView(variant val);     // change minimal panel view to save in config
    signal commandMoveItem(variant from, variant to);     // move playlist item
    signal commandSaveAll();                            // save all button action
    signal commandMoved(variant x, variant y); //signal on mouse moving (only for linux)
    signal commandPlayOnClick();                                   // play button click
    signal commandSetQuality(variant val);

    signal skipAd();
    signal unableAds();
    signal visitAd();
    
    function changedIsAd(is_ad) { root.isAd = is_ad; }
    function changedWaitForSkip(tm) { root.waitForAd = tm; }
    function changedAdCanBeDisabled(canDis) { root.adCanBeDisabled = canDis; }
    function changedAdCanBeVisited(canDis) { root.adCanBeVisited = canDis; }
    function changedVisitText(vText) { root.visittext = vText; }
    /* slots */
    /* state changed stoped->playing or paused->playing */
    function changedPlaying() { panel.state = "playing"; }
    /* state changed playing->paused */
    function changedPaused() { panel.state = "paused"; }
    /* state changed to stopped or fullstopped */
    function changedStopped(isFullstop) { panel.state = isFullstop ? "fullstopped" : "stopped"; }
    /* mute value changed (bool) */
    function changedMute(mute) { panel.isMute = mute; }
    /* playback value changed (0<=double<=1) */
    function changedPlayback(playback) { if( panel.state == "stopped" || panel.state == "fullstopped" ) return; panel.playback = playback; }
    /* duration value changed (string format xx:xx:xx) */
    function changedDuration(durationString) { panel.duration = Lib.stringAsSeconds(durationString); }
    /* live stream playing (bool) */
    function changedLiveStream(isLiveStream) { panel.isLiveStream = isLiveStream; }
    /* title of playing item (string) */
    function changedMediaTitle(title) { panel.title = title; }
    /* status message (string) */
    function changedStatus(status) { panel.infoMsg = status; }
    /* volume changed (0<=int<=100) */
    function changedVolume(volume) { panel.volume = volume; }
    /* if playlist items count > 1 -> has playlist */
    function changedHasPlaylist(haspl) { panel.hasPlaylist = haspl; }
    /* wheel event on holder window after commandMouseOnPlaylist(true) */
    function playlistWheel(val) { playlist.wheel(val); }
    /* hide playlist */
    function playlistHide() { playlist.visible = false; }
    /* begin show animation */
    function controlsShow() { if(panel.opacity==0) animationShow.start(); }
    /* begin hide animation */
    function controlsHide() { if(root.canHide && panel.opacity>0) animationHide.start() }
    /* begin stop animation while running*/
    function controlsReanimateIfRunning() {
        if(animationHide.running) {
            animationHide.forced = true;
            animationHide.stop();
            panel.opacity = 0.9;
        }
        else
            animationHide.forced = false;
    }
    /* current playing item is saveable */
    function changedSaveable(val) { panel.isSaveable = val; }
    function changedLiveStreamIsLive(val) { if( panel.state == "stopped" || panel.state == "fullstopped" ) return; panel.liveStreamIsLive = val; }
    function changedLiveStreamBufferPos(val) { if( panel.state == "stopped" || panel.state == "fullstopped" ) return; panel.liveStreamBufferPos = val < panel.playback ? panel.playback : val; }
    function changedQualities(val1, val2) { if(val1 == "") return; panel.qualities=val1.split("|"); panel.bitrates=val2.split("|"); }
    function changedQuality(idx) { if(panel.qualities == 0 ) return; panel.quality= idx; }

    onClicked: commandPlayOnClick()
    onPositionChanged: commandMoved(mouseX, mouseY)

    /* translator */
    TranslatorObject {
        id: translator
    }
    function translate(val) { return translator.translate(val); }

    /* default/small panel switcher */
    DoubleStateButton {
        id: btnPanelSwitcher
        anchors.horizontalCenter: root.horizontalCenter
        anchors.bottom: root.bottom
        anchors.bottomMargin: panel.height + Math.round(panel.width * 0.004)
        visible: !root.isAd
        pixmaps: {
            'default1': root.imgPath+"squeeze.png",
            'hovered1': root.imgPath+"squeeze_h.png",
            'default2': root.imgPath+"expand.png",
            'hovered2': root.imgPath+"expand_h.png"
        }
        condition: panel.isMinimalView
        onClicked: { panel.isMinimalView = !panel.isMinimalView; root.commandSetMinView(panel.isMinimalView) }
    }

    Rectangle {
        width: root.width
        height: 1
        color: "#000000"
        anchors.bottom: root.bottom
    }
    
    Rectangle {
        id: btnVisitAdSite
        anchors.horizontalCenter: root.horizontalCenter
        anchors.bottom: root.bottom
        anchors.bottomMargin: panel.height + Math.round(panel.width * 0.004)
        visible: root.isAd && root.adCanBeVisited
        width: lblVisit.width + 10
        height: 26
        color: "#444444"

        MouseArea {
            id: txt_layout
            anchors.fill: parent
            hoverEnabled: true

            onClicked: root.visitAd();

            Text {
                id: lblVisit
                text: root.visittext
                color: txt_layout.containsMouse ? "#D5D5D5" : "#E5E5E5"
                font.pixelSize: 12
                anchors.centerIn: parent
            }

            CursorShapeArea {
                anchors.fill: parent
                cursorShape: Qt.PointingHandCursor
            }
        }
    }
    
    /* panel from Panel.qml */
    Panel {
        id: panel
        width: root.width
        height: isMinimalView || root.isAd ? 48 : root.panelSize.height
        anchors.bottom: root.bottom

        onEntered: root.commandCanHideChanged(false)
        onExited: root.commandCanHideChanged(!playlist.visible)

        NumberAnimation {
            id: animationShow
            target: panel
            properties: "opacity"
            from: 0.0
            to: 1
            duration: 500
        }
        NumberAnimation {
            id: animationHide
            target: panel
            properties: "opacity"
            from: 1
            to: 0.0
            duration: 500
            property bool forced: false
            onRunningChanged: { if(!running && !forced) root.commandHidden() }
        }
    }

    /* playlist from Playlist.qml */
    Playlist {
        id: playlist
        x: (root.width - width) / 2
        y: (root.height - panel.height - height) / 2
        width: root.playlistSize.width
        height: root.playlistSize.height
        visible: false
    }
}

