import QtQuick 1.1
import "functions.js" as Lib
import CursorShape 1.0
import Translator 1.0
import StandardToolTip 1.0

/* panel.qml */
/* background image */
Image {
    id: root
    width: 600
    height: 36
    source: "panel/bg.png"
    fillMode: Image.TileHorizontally
    smooth: true
    state: "stopped"

    property double playback: .0                     // playback value
    property int volume: 0                           // volume value
    property int duration: 0                         // duration value
    property bool isLiveStream: false                // live stream flag
    property bool hasPlaylist: false                 // playlist has items flag
    property bool isMute: false                      // mute value
    property string title: ""                        // current media title
    property string infoMsg: ""                      // info msg
    property bool isSaveable: false                  // if saveable, save button is visible
    property bool isPlaylistVisible: false           // playlist visiability
    property int liveStreamIsLive: -1
    property double liveStreamBufferPos: 1.0
    property bool hasQualityList: qualities.length > 0
    property variant qualities: 0                    // qualities array
    property variant bitrates: 0                     // bitrates array
    property int quality: 0                          // current quality index
    property bool isAd: false
    property int waitForAd: 0
    property bool adCanBeDisabled: false
    
    
    /* signals */
    signal commandPlay();                            // play button click
    signal commandStop(variant val);                 // stop/power button click
    signal commandMute();                            // mute button click
    signal commandFullscreen();                      // fullscreen button click
    signal commandVolumeSet(variant val);            // volume value change
    signal commandPlaybackSet(variant val);          // playback value change
    signal commandSaveCurrent();                     // save button on panel clicked
    signal commandTogglePlaylist();                  // show/hide playlist
    signal commandPrev();                            // prev button click
    signal commandNext();                            // next button click
    signal commandSetQuality(variant val);
    
    signal skipAd();
    signal unableAds();

    function changedIsAd(is_ad) { root.isAd = is_ad; }
    function changedWaitForSkip(tm) { root.waitForAd = tm; }
    function changedAdCanBeDisabled(canDis) { root.adCanBeDisabled = canDis; }
    /* slots */
    /* state changed stoped->playing or paused->playing */
    function changedPlaying() { root.state = "playing"; }
    /* state changed playing->paused */
    function changedPaused() { root.state = "paused"; }
    /* state changed to stopped or fullstopped */
    function changedStopped(isFullstop) { root.state = isFullstop ? "fullstopped" : "stopped"; }
    /* mute value changed (bool) */
    function changedMute(mute) { root.isMute = mute; }
    /* playback value changed (0<=double<=1) */
    function changedPlayback(playback) { if( root.state == "stopped" || root.state == "fullstopped" ) return; root.playback = playback; }
    /* duration value changed (string format xx:xx:xx) */
    function changedDuration(durationString) { root.duration = Lib.stringAsSeconds(durationString); }
    /* live stream playing (bool) */
    function changedLiveStream(isLiveStream) { root.isLiveStream = isLiveStream; }
    /* title of playing item (string) */
    function changedMediaTitle(title) { root.title = title; }
    /* status message (string) */
    function changedStatus(status) { if(!volumeScaleArea.containsMouse && !playbackProgressBarArea.containsMouse) root.infoMsg = status; }
    /* volume changed (0<=int<=100) */
    function changedVolume(volume) { root.volume = volume; }
    /* if playlist items count > 1 -> has playlist */
    function changedHasPlaylist(haspl) { root.hasPlaylist = haspl; }
    /* current playing item is saveable */
    function changedSaveable(val) { root.isSaveable = val; }
    /* playlist visible changed */
    function changedPlaylistVisible(val) { root.isPlaylistVisible = val; }
    function changedLiveStreamIsLive(val) { if( root.state == "stopped" || root.state == "fullstopped" ) return; root.liveStreamIsLive = val; }
    function changedLiveStreamBufferPos(val) { if( root.state == "stopped" || root.state == "fullstopped" ) return; root.liveStreamBufferPos = val < root.playback ? root.playback : val; }
    function changedQualities(val1, val2) { if(val1 == "") return; root.qualities=val1.split("|"); root.bitrates=val2.split("|"); }
    function changedQuality(idx) { if(root.qualities == 0 ) return; root.quality=idx; }
    
    /* menu quality functions */
    function qualityButtonWidth() {
        var q = root.qualities;
        q.sort(root.sortFunction);
        var max_str = q[0]
        var newObject = Qt.createQmlObject('import QtQuick 1.0;' +
            'Text {text: "'+max_str+'"; visible:false; font.pixelSize: 12}', root);
        var width_q = newObject.width;
        newObject.destroy();

        var b = root.bitrates;
        b.sort(root.sortFunction);
        max_str = b[0]
        var newObject1 = Qt.createQmlObject('import QtQuick 1.0;' +
            'Text {text: "'+max_str+'"; visible:false; font.pixelSize: 8}', root);
        var width_b = newObject1.width;
        newObject1.destroy();

        var width = width_q > width_b ? width_q : width_b
        return width + 10;
    }

    function sortFunction(a, b) {
        if(a.length > b.length)
            return -1;
        if(a.length < b.lenght)
            return 1;
        return 0;
    }

    function initQualityMenu() {
        var itemWidth = root.qualityButtonWidth();
        for(var i = 0; i < root.qualities.length; i++) {
            rowQuality.children[i+1].width = itemWidth;
            rowQuality.children[i+1].name = root.qualities[i];
            rowQuality.children[i+1].bitrate =root.bitrates[i];
            rowQuality.children[i+1].hovered = false;
            rowQuality.children[i+1].visible = true;
        }
        menuQuality.visible = true;
    }

    function uninitQualityMenu() {
        menuQuality.visible = false;
        for(var i = 0; i < menuQuality.itemsCount; i++) {
            rowQuality.children[i+1].hovered = false;
            rowQuality.children[i+1].visible = false;
        }
    }

    states: [
        State {
            name: "playing"
        },
        State {
            name: "paused"
        },
        State {
            name: "stopped"
            PropertyChanges {
                target: root
                playback: .0
                duration: 0
                liveStreamBufferPos: 1.0
                liveStreamIsLive: -1
                title: ""
                restoreEntryValues: false
                //qualities: 0
                //bitrates: 0
            }
        },
        State {
            name: "fullstopped"
            PropertyChanges {
                target: root
                playback: .0
                duration: 0
                liveStreamBufferPos: 1.0
                liveStreamIsLive: -1
                title: ""
                restoreEntryValues: false
                //qualities: 0
                //bitrates: 0
            }
        }
    ]

    /* translator */
    TranslatorObject {
        id: translator
    }
    function translate(val) { return translator.translate(val); }

    /* panel view */
    Column {
        id: panelMainLayout
        anchors.fill: parent
        spacing: 0

        /* playback progress bar */
        Rectangle {
            id: playbackProgressBar
            height: 8
            anchors.left: parent.left
            anchors.right: parent.right
            color: "#676767"

            MouseArea {
                id: playbackProgressBarArea
                anchors.fill: parent
                hoverEnabled: true

                onClicked: {
                    if( root.state == "stopped" || root.state == "fullstopped" /*|| root.isLiveStream*/ ) return;
                    var newPlayback = mouseX / playbackProgressBar.width;
                    if( root.isLiveStream )
                        root.infoMsg = "-" + Lib.secondsAsString(parseInt(root.duration - root.duration * newPlayback));
                    else
                        root.infoMsg = Lib.secondsAsString(parseInt(root.duration * newPlayback));
                    root.playback = newPlayback;
                    root.commandPlaybackSet(newPlayback);
                }
                onMousePositionChanged: {
                    if( root.state == "stopped" || root.state == "fullstopped" /*|| root.isLiveStream*/ ) return;
                    var newPlayback = mouseX / playbackProgressBar.width;
                    if( root.isLiveStream )
                        root.infoMsg = "-" + Lib.secondsAsString(parseInt(root.duration - root.duration * newPlayback));
                    else
                        root.infoMsg = Lib.secondsAsString(parseInt(root.duration * newPlayback));
                }
                onExited: root.infoMsg = ""

                /* playback progress bar value */
                Rectangle {
                    id: playbackProgressBarValue
                    width: root.playback * parent.width
                    height: parent.height
                    opacity: 1
                    color: root.isLiveStream ? "#676767" : "#498C9F"
                }

                /* playback live indicator */
                Rectangle {
                    id: playbackProgressBarLiveIndicator
                    x: parent.width - width
                    width: parent.width - playbackProgressBarValue.width
                    height: parent.height
                    opacity: 1
                    color: root.liveStreamIsLive == -1 ? "#676767" : root.isLiveStream ? "#dd9a22" : "#676767"
                }

                /* buffer live indicator */
                Rectangle {
                    id: playbackProgressBarLiveBufferIndicator
                    x: parent.width - width
                    width: parent.width - root.liveStreamBufferPos * parent.width
                    height: parent.height
                    opacity: 1
                    color: root.liveStreamIsLive == -1 ? "#676767" : root.isLiveStream ? "#498C9F" : "#676767"
                }

                /* playback progress bar slider */
                Rectangle {
                    id: playbackProgressBarSlider
                    x: playbackProgressBarSliderArea.drag.active ? x : playbackProgressBarValue.width - width / 2
                    width: height
                    height: parent.height
                    color: "#e5e5e5"

                    MouseArea {
                        id: playbackProgressBarSliderArea
                        anchors.fill: parent
                        drag.target: parent
                        drag.axis: Drag.XAxis
                        drag.minimumX: 0 - playbackProgressBarSlider.width / 2
                        drag.maximumX: ( root.state == "stopped" || root.state == "fullstopped" /*|| root.isLiveStream*/ ) ? drag.minimumX : playbackProgressBar.width - playbackProgressBarSlider.width / 2

                        onPositionChanged: {
                            if( root.state == "stopped" || root.state == "fullstopped" /*|| root.isLiveStream*/ ) return;
                            var newPlayback = (playbackProgressBarSlider.x + playbackProgressBarSlider.width / 2) / playbackProgressBar.width;
                            if( root.isLiveStream )
                                root.infoMsg = "-" + Lib.secondsAsString(parseInt(root.duration - root.duration * newPlayback));
                            else
                                root.infoMsg = Lib.secondsAsString(parseInt(root.duration * newPlayback));
                        }

                        onReleased: {
                            if( root.state == "stopped" || root.state == "fullstopped" /*|| root.isLiveStream*/ ) return;
                            var newPlayback = (playbackProgressBarSlider.x + playbackProgressBarSlider.width / 2) / playbackProgressBar.width;
                            root.playback = newPlayback;
                            root.commandPlaybackSet(newPlayback);
                        }
                    }
                }
            }
        }

        /* panel controls */
        Item {
            id: panel
            height: root.height - playbackProgressBar.height
            anchors.left: parent.left
            anchors.right: parent.right

            /* qualities menu */
            MouseArea {
                id: menuQuality
                hoverEnabled: true
                z: 5
                //x: rightRow.x - rowQuality.width / 2 + btnQuality.width / 2
                anchors.right: parent.right
                anchors.rightMargin: rowQuality.width + btnPower.width
                height: panel.height
                visible: false
                onExited: root.uninitQualityMenu()
                property int itemsCount: 5

                Row {
                    id: rowQuality
                    height: parent.height

                    Rectangle {
                        width: 1
                        height: parent.height
                        color: "#404040"
                    }

                    Image {
                        width: 0
                        height: panel.height;
                        source: "panel/bg.png";
                        property string name: "";
                        property string bitrate: "";
                        property int index: 0;
                        property bool hovered: false

                        /*Rectangle {
                            anchors.fill: parent
                            color: parent.index == root.quality ? "#404040" : "transparent"
                        }*/

                        Text {
                            text: parent.name
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 12
                            anchors.top: parent.top
                            anchors.topMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Text {
                            text: parent.bitrate
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 8
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Rectangle {
                            width: 1
                            height: parent.height
                            anchors.right: parent.right
                            color: "#404040"
                        }

                        MouseArea {
                            hoverEnabled: true
                            anchors.fill: parent
                            onClicked: {
                                root.uninitQualityMenu();
                                if(parent.index != root.quality)
                                    root.commandSetQuality(parent.index);
                                root.quality = parent.name;
                                root.bitrate = parent.bitrate;
                            }
                            onEntered: parent.hovered = true
                            onExited: parent.hovered = false
                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }

                            StandardToolTipObject {
                                anchors.fill: parent
                                text: parent.name
                            }
                        }
                    }

                    Image {
                        width: 0
                        height: panel.height;
                        source: "panel/bg.png";
                        property string name: "";
                        property string bitrate: "";
                        property int index: 1;
                        property bool hovered: false

                        Text {
                            text: parent.name
                            color: parent.index == root.quality? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 12
                            anchors.top: parent.top
                            anchors.topMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Text {
                            text: parent.bitrate
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 8
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Rectangle {
                            width: 1
                            height: parent.height
                            anchors.right: parent.right
                            color: "#404040"
                        }

                        MouseArea {
                            hoverEnabled: true
                            anchors.fill: parent
                            onClicked: {
                                root.uninitQualityMenu();
                                if(parent.index != root.quality)
                                    root.commandSetQuality(parent.index);
                                root.quality = parent.name;
                                root.bitrate = parent.bitrate;
                            }
                            onEntered: parent.hovered = true
                            onExited: parent.hovered = false
                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }

                            StandardToolTipObject {
                                anchors.fill: parent
                                text: parent.name
                            }
                        }
                    }

                    Image {
                        width: 0
                        height: panel.height;
                        source: "panel/bg.png";
                        property string name: "";
                        property string bitrate: "";
                        property int index: 2;
                        property bool hovered: false

                        Text {
                            text: parent.name
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 12
                            anchors.top: parent.top
                            anchors.topMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Text {
                            text: parent.bitrate
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 8
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Rectangle {
                            width: 1
                            height: parent.height
                            anchors.right: parent.right
                            color: "#404040"
                        }

                        MouseArea {
                            hoverEnabled: true
                            anchors.fill: parent
                            onClicked: {
                                root.uninitQualityMenu();
                                if(parent.index != root.quality)
                                    root.commandSetQuality(parent.index);
                                root.quality = parent.name;
                                root.bitrate = parent.bitrate;
                            }
                            onEntered: parent.hovered = true
                            onExited: parent.hovered = false
                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }

                            StandardToolTipObject {
                                anchors.fill: parent
                                text: parent.name
                            }
                        }
                    }

                    Image {
                        width: 0
                        height: panel.height;
                        source: "panel/bg.png";
                        property string name: "";
                        property string bitrate: "";
                        property int index: 3;
                        property bool hovered: false

                        Text {
                            text: parent.name
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 12
                            anchors.top: parent.top
                            anchors.topMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Text {
                            text: parent.bitrate
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 8
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Rectangle {
                            width: 1
                            height: parent.height
                            anchors.right: parent.right
                            color: "#404040"
                        }

                        MouseArea {
                            hoverEnabled: true
                            anchors.fill: parent
                            onClicked: {
                                root.uninitQualityMenu();
                                if(parent.index != root.quality)
                                    root.commandSetQuality(parent.index);
                                root.quality = parent.name;
                                root.bitrate = parent.bitrate;
                            }
                            onEntered: parent.hovered = true
                            onExited: parent.hovered = false
                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }

                            StandardToolTipObject {
                                anchors.fill: parent
                                text: parent.name
                            }
                        }
                    }

                    Image {
                        width: 0
                        height: panel.height;
                        source: "panel/bg.png";
                        property string name: "";
                        property string bitrate: "";
                        property int index: 4;
                        property bool hovered: false

                        Text {
                            text: parent.name
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 12
                            anchors.top: parent.top
                            anchors.topMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Text {
                            text: parent.bitrate
                            color: parent.index == root.quality ? "#00a691" : parent.hovered ? "#ffffff" : "#909090"
                            font.pixelSize: 8
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 1
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Rectangle {
                            width: 1
                            height: parent.height
                            anchors.right: parent.right
                            color: "#404040"
                        }

                        MouseArea {
                            hoverEnabled: true
                            anchors.fill: parent
                            onClicked: {
                                root.uninitQualityMenu();
                                if(parent.index != root.quality)
                                    root.commandSetQuality(parent.index);
                                root.quality = parent.name;
                                root.bitrate = parent.bitrate;
                            }
                            onEntered: parent.hovered = true
                            onExited: parent.hovered = false
                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }

                            StandardToolTipObject {
                                anchors.fill: parent
                                text: parent.name
                            }
                        }
                    }
                }
            }

            /* left side buttons layout */
            Row {
                id: leftRow
                spacing: 0

                /* skip ad button */
                    Item {
                        id: btnAd
                        width: state == "wait" ? lblAd.width + 10 + parent.height : lblSkip.width + 10
                        height: parent.height
                        visible: root.isAd

                        states: [
                            State {
                                name: "wait"
                                when: root.isAd && root.waitForAd >= 0
                                PropertyChanges {
                                    target: skip_layout
                                    visible: false
                                }
                                PropertyChanges {
                                    target: wait_layout
                                    visible: true
                                }
                            },
                            State {
                                name: "skip"
                                when: root.isAd && root.waitForAd < 0
                                PropertyChanges {
                                    target: skip_layout
                                    visible: true
                                }
                                PropertyChanges {
                                    target: wait_layout
                                    visible: false
                                }
                            }
                        ]

                        MouseArea {
                            id: wait_layout
                            anchors.fill: parent
                            visible: false

                            onClicked: {}

                            Item {
                                width: parent.height
                                height: parent.height
                                anchors.top: parent.top
                                anchors.left: parent.left

                                Image {
                                    source: "panel/ad_waiting.png"
                                    anchors.fill: parent
                                    smooth: true

                                    NumberAnimation on rotation {
                                        from: 0;
                                        to: 360;
                                        running: visible;
                                        loops: Animation.Infinite;
                                        duration: 900
                                    }
                                }

                                Text {
                                    id: time_wait
                                    text: root.waitForAd
                                    font.pixelSize: 12
                                    color: "#E5E5E5"
                                    anchors.centerIn: parent
                                }
                            }

                            Item {
                                width: lblAd.width + 10
                                height: parent.height
                                anchors.top: parent.top
                                anchors.right: parent.right

                                Text {
                                    id: lblAd
                                    text: translator.translate("Advertising")
                                    font.pixelSize: 12
                                    anchors.centerIn: parent
                                    color: "#E5E5E5"
                                }
                            }
                        }

                        MouseArea {
                            id: skip_layout
                            anchors.fill: parent
                            hoverEnabled: true
                            visible: false

                            onClicked: root.skipAd();

                            Text {
                                id: lblSkip
                                text: translator.translate("Skip this ad")
                                color: skip_layout.containsMouse ? "#D5D5D5" : "#E5E5E5"
                                font.pixelSize: 12
                                anchors.centerIn: parent
                            }

                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }
                        }
                    }

                    /* separator */
                    Rectangle {
                        width: 1
                        height: panel.height
                        color: "#404040"
                        visible: btnAd.visible
                    }
                
                /* prev buttton */
                Button {
                    id: btnPrev
                    width: 55
                    height: panel.height
                    anchors.verticalCenter: parent.verticalCenter
                    visible: !root.isLiveStream && root.hasPlaylist && !root.isAd
                    pixmaps: {
                        'default': "panel/prev.png",
                        'hovered': "panel/prev_h.png"
                    }
                    tooltip: "Previous"
                    onClicked: root.commandPrev()
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnPrev.visible
                }

                /* play button */
                DoubleStateButton {
                    id: btnPlay
                    width: 55
                    height: panel.height
                    anchors.verticalCenter: parent.verticalCenter
                    pixmaps: {
                        'default1': "panel/play-btn.png",
                        'hovered1': "panel/play-btn_h.png",
                        'default2': "panel/pause-btn.png",
                        'hovered2': "panel/pause-btn_h.png"
                    }
                    tooltips: {
                        '1': "Play",
                        '2': "Pause"
                    }
                    visible: !root.isAd

                    condition: root.state=="playing"
                    onClicked: root.commandPlay()
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnPlay.visible
                }

                /* next button */
                Button {
                    id: btnNext
                    width: 55
                    height: panel.height
                    anchors.verticalCenter: parent.verticalCenter
                    visible: !root.isLiveStream && root.hasPlaylist && !root.isAd
                    pixmaps: {
                        'default': "panel/next.png",
                        'hovered': "panel/next_h.png"
                    }
                    tooltip: "Next"
                    onClicked: root.commandNext()
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnNext.visible
                }
                
                /* Live */
                Item {
                    id: live_options
                    width: live_name.width+live_name.x+live_img.x*2 //56
                    height: parent.height
                    y: 0
                    x: volume_holder.x + volume_holder.width
                    visible: root.isLiveStream && !root.isAd

                    Image {
                        id: live_img
                        width: 10
                        height: 10
                        x: 9
                        y: 8
                        smooth: true
                        
                        states: [
                            State {
                                name: "undefined"
                                when: root.liveStreamIsLive == -1
                                PropertyChanges { target: live_img; source: "panel/live_no.png" }
                            },
                            State {
                                name: "timeshiftable"
                                when: root.liveStreamIsLive == 0
                                PropertyChanges { target: live_img; source: "panel/live_time.png" }
                            },
                            State {
                                name: "untimeshiftable"
                                when: root.liveStreamIsLive == 1
                                PropertyChanges { target: live_img; source: "panel/live.png" }
                            }
                        ]
                        
                        Text {
                            id: live_name
                            text: root.translate("Live")
                            color: "#909090"
                            font.pixelSize: 12
                            x: parent.width + 7
                            y: -2
                        }
                    }

                    MouseArea {
                        id: live_area
                        anchors.fill: parent
                        
                        CursorShapeArea {
                            anchors.fill: parent
                            cursorShape: live_img.state == "timeshiftable" ? Qt.PointingHandCursor : Qt.ArrowCursor
                        }

                        StandardToolTipObject {
                            anchors.fill: parent
                            text: {
                                if(live_img.state == "untimeshiftable") {
                                    return "You are watching live broadcast";
                                }
                                else {
                                    return "Skip ahead to live broadcast";
                                }
                            }
                        }
                        
                        onClicked: if( live_img.state == "timeshiftable" ) root.commandPlaybackSet(-1)
                    }
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: live_options.visible
                }

                /* volume controls */
                Item {
                    id: volumeHolder
                    width:  volumeControlsRow.width
                    height: panel.height

                    MouseArea {
                        id: volumeHolderArea
                        anchors.fill: parent
                        hoverEnabled: true

                        Row {
                            id: volumeControlsRow
                            spacing: 0

                            /* mute button */
                            DoubleStateButton {
                                id: btnMute
                                width: 33
                                height: panel.height
                                anchors.verticalCenter: parent.verticalCenter
                                pixmaps: {
                                    'default1': "panel/mute-btn-on.png",
                                    'hovered1': "panel/mute-btn-on_h.png",
                                    'default2': "panel/mute-btn-off.png",
                                    'hovered2': "panel/mute-btn-off_h.png"
                                }
                                condition: root.isMute
                                tooltips: {
                                     '1': "Mute on",
                                     '2': "Mute off"
                                }
                                onClicked: root.commandMute()
                            }

                            Row {
                                spacing: 10

                                /* volume scale */
                                Item {
                                    id: volumeScaleHolder
                                    width: 108
                                    height: panel.height
                                    visible: volumeHolderArea.containsMouse

                                    Image {
                                        id: volumeScale
                                        width: parent.width
                                        height: 12
                                        anchors.verticalCenter: parent.verticalCenter
                                        anchors.horizontalCenter: parent.horizontalCenter
                                        source: "panel/scale.png"

                                        Rectangle {
                                            id: soundLevel
                                            anchors.verticalCenter: parent.verticalCenter
                                            anchors.horizontalCenter: parent.horizontalCenter
                                            width: parent.width
                                            height: 4
                                            color: "#404040"

                                            Rectangle {
                                                x: 0
                                                width: root.volume * parent.width / 100
                                                height: parent.height
                                                color: "#00a691"
                                            }
                                            Rectangle {
                                                x: parent.width * 0.58
                                                width: (root.volume - 58) * parent.width / 100
                                                height: parent.height
                                                color: "#dd9a22"
                                                visible: root.volume > 58
                                            }
                                            Rectangle {
                                                x: parent.width * 0.76
                                                width: (root.volume - 76) * parent.width / 100
                                                height: parent.height
                                                color: "#d73e3e"
                                                visible: root.volume > 76
                                            }
                                        }

                                        MouseArea {
                                            id: volumeScaleArea
                                            anchors.fill: parent
                                            hoverEnabled: true
                                            onClicked: {
                                                var newVolume = parseInt(100 * mouseX / volumeScale.width);
                                                root.commandVolumeSet(newVolume);
                                            }
                                            onMousePositionChanged: {
                                                var newVolume = parseInt(100 * mouseX / volumeScale.width);
                                                root.infoMsg = newVolume + "%";
                                            }
                                            onExited: root.infoMsg = ""
                                        }

                                        Rectangle {
                                            id: volumeSlider
                                            x: volumeSliderArea.drag.active ? x : root.volume * volumeScale.width / 100
                                            y: (parent.height - height) / 2
                                            width: 2
                                            height: Math.floor(parent.height*0.8462)
                                            color: "#f8f8f8"

                                            MouseArea {
                                                id: volumeSliderArea
                                                anchors.fill: parent
                                                drag.target: parent
                                                drag.axis: Drag.XAxis
                                                drag.minimumX: 0 - volumeSlider.width / 2
                                                drag.maximumX: volumeScale.width - volumeSlider.width / 2

                                                onPositionChanged: {
                                                    var newVolume = parseInt(100 * (volumeSlider.x + volumeSlider.width / 2) / volumeScale.width);
                                                    root.infoMsg = newVolume + "%";
                                                    root.commandVolumeSet(newVolume);
                                                }
                                            }
                                        }
                                    }
                                }

                                /* separator */
                                Rectangle {
                                    width: 1
                                    height: panel.height
                                    color: "#404040"
                                    visible: btnMute.visible
                                }
                            }
                        }
                    }
                }

                /* playback/duration label */
                Item {
                    id: lblTimeValues
                    width: lblTimeLayout.width + 10
                    height: panel.height
                    visible: !root.isLiveStream && !volumeHolderArea.containsMouse && !root.isAd

                    Row {
                        id: lblTimeLayout
                        anchors.left: parent.left
                        anchors.leftMargin: 5
                        anchors.verticalCenter: parent.verticalCenter
                        spacing: 0

                        Text {
                            id: lblPlayback
                            text: Lib.secondsAsString(parseInt(root.duration * root.playback))
                            font.pixelSize: 11
                            color: "#e5e5e5"
                            smooth: true
                        }
                        Text {
                            text: "/"
                            font.pixelSize: 11
                            color: "#e5e5e5"
                            smooth: true
                        }
                        Text {
                            id: lblDuration
                            text: Lib.secondsAsString(root.duration)
                            font.pixelSize: 11
                            color: "#e5e5e5"
                            smooth: true
                        }
                    }
                }
            }

            /* information label */
            Item {
                id: lblInformationItem
                height: panel.height
                anchors.left: leftRow.right
                anchors.leftMargin: 5
                anchors.right: rightRow.left
                anchors.rightMargin: 5
                anchors.verticalCenter: parent.verticalCenter
                clip: true

                Text {
                    id: lblInfo
                    anchors.verticalCenter: parent.verticalCenter
                    font.pixelSize: 11
                    color: "#e5e5e5"
                    text: root.infoMsg == "" ? root.title : root.infoMsg
                    property bool animate:  lblInfo.width / lblInformationItem.width > 0.8

                    onTextChanged: {
                        animate = lblInfo.width / lblInformationItem.width > 0.8
                        if(animate) {
                            if(!animateInfo.running) {
                                animateInfo.from = 0;
                                animateInfo.to = -lblInfo.width;
                                animateInfo.duration = 20000;
                                animateInfo.start();
                            }
                        }
                        else {
                            animateInfo.stop();
                            lblInfo.x=0;
                        }
                    }

                    PropertyAnimation on x {
                        id: animateInfo
                        running: false
                        from: 0;
                        to: -lblInfo.width;
                        duration: 20000;
                        onRunningChanged: {
                            if(!running && lblInfo.animate ) {
                                if( from === 0 ) {
                                    from = lblInformationItem.width;
                                    duration = duration * 1.5;
                                }
                                start()
                            }
                        }
                    }
                }

                MouseArea {
                    anchors.fill: parent
                    hoverEnabled: true
                    onEntered: { if(lblInfo.animate) animateInfo.pause() }
                    onExited: { if(lblInfo.animate) animateInfo.resume() }
                }
            }

            /* right side buttons layout */
            Row {
                id: rightRow
                anchors.right: parent.right
                spacing: 0

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnQuality.visible
                }

                /* quality button */
                Item {
                    id: btnQuality
                    visible: root.hasQualityList && !root.isAd
                    width: Math.max(lblQuality.width, lblBitrate.width) + 10
                    height: panel.height

                    Text {
                        id: lblQuality
                        text: root.qualities[root.quality]
                        color: "#909090"
                        font.pixelSize: 12
                        anchors.top: parent.top
                        anchors.topMargin: 1
                        anchors.horizontalCenter: parent.horizontalCenter
                    }

                    Text {
                        id: lblBitrate
                        text: root.bitrates[root.quality]
                        color: "#909090"
                        font.pixelSize: 8
                        anchors.bottom: parent.bottom
                        anchors.bottomMargin: 1
                        anchors.horizontalCenter: parent.horizontalCenter
                    }

                    MouseArea {
                        anchors.fill: parent
                        hoverEnabled: true
                        onClicked: root.initQualityMenu()

                        CursorShapeArea {
                            anchors.fill: parent
                            cursorShape: Qt.PointingHandCursor
                        }

                        StandardToolTipObject {
                            anchors.fill: parent
                            text: root.translate("Quality menu")
                        }
                    }
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnSaveCurrent.visible
                }

                /* save button */
                Button {
                    id: btnSaveCurrent
                    width: 55
                    height: panel.height
                    visible: root.isSaveable && !root.isAd
                    pixmaps: {
                        'default': "panel/save-btn.png",
                        'hovered': "panel/save-btn_h.png"
                    }
                    tooltip: "Save"
                    onClicked: root.commandSaveCurrent()
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnPlaylist.visible
                }

                /* playlist button */
                DoubleStateButton {
                    id: btnPlaylist
                    width: 55
                    height: panel.height
                    anchors.verticalCenter: parent.verticalCenter
                    visible: root.hasPlaylist && !root.isAd//!root.isLiveStream
                    pixmaps: {
                        'default1': "panel/playlist-btn.png",
                        'hovered1': "panel/playlist-btn_h.png",
                        'default2': "panel/playlist-active.png",
                        'hovered2': "panel/playlist-active.png"
                    }
                    condition: root.isPlaylistVisible
                    tooltips: {
                        '1': "Playlist",
                        '2': "Playlist"
                    }

                    onClicked: root.commandTogglePlaylist()
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnFullscreen.visible
                }

                /* fullscreen button */
                Button {
                    id: btnFullscreen
                    width: 55
                    height: panel.height
                    anchors.verticalCenter: parent.verticalCenter
                    pixmaps: {
                        'default': "panel/fullscreen-btn.png",
                        'hovered': "panel/fullscreen-btn_h.png"
                    }
                    tooltip: "Fullscreen"
                    onClicked: root.commandFullscreen()
                }

                /* separator */
                Rectangle {
                    width: 1
                    height: panel.height
                    color: "#404040"
                    visible: btnPower.visible
                }

                /* power button */
                DoubleStateButton {
                    id: btnPower
                    width: 55
                    height: panel.height
                    anchors.verticalCenter: parent.verticalCenter
                    visible: !root.isAd
                    pixmaps: {
                        'default1': "panel/power-on.png",
                        'hovered1': "panel/power-on_h.png",
                        'default2': "panel/power-off.png",
                        'hovered2': "panel/power-off_h.png"
                    }
                    condition: root.state=="fullstopped"
                    tooltips: {
                         '1': "Turn off",
                         '2': "Turn off"
                    }

                    onClicked: { root.state="fullstopped"; root.commandStop(true); }
                }
                
                 /* separator */
                    Rectangle {
                        width: 1
                        height: panel.height
                        color: "#404040"
                        visible: btnUnableAds.visible
                    }

                    /* unable ads button */
                    Item {
                        id: btnUnableAds
                        visible: root.isAd && root.adCanBeDisabled
                        width: lblUnableAds.width + 10
                        height: panel.height

                        Text {
                            id: lblUnableAds
                            text: translator.translate("Disable ads")
                            color: btnUnableAdsArea.containsMouse ? "#D5D5D5" : "#E5E5E5"
                            font.pixelSize: 12
                            anchors.centerIn: parent
                        }

                        MouseArea {
                            id: btnUnableAdsArea
                            anchors.fill: parent
                            hoverEnabled: true
                            onClicked: root.unableAds()

                            CursorShapeArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                            }

                            StandardToolTipObject {
                                anchors.fill: parent
                                text: root.translate("Do not display ads anymore")
                            }
                        }
                    }
            }
        }
    }
}
